import paramiko,time
if __name__ == '__main__':
    host = "10.10.70.1"
    port = 22
    username = "root"
    password = ""

    # sshClient = paramiko.SSHClient()
    # sshClient.connect(hostname=host, username=username, password=password)
    # channel = sshClient.get_transport().open_session()
    # channel.get_pty()
    # channel.invoke_shell()

    # while True:
    #     command = input('$ ')
    #     if command == 'exit':
    #         break

    #     channel.send(command + "\n")

    #     while True:
    #         if channel.recv_ready():
    #             output = channel.recv(1024)
    #             print(output)
    #         else:
    #             time.sleep(0.5)
    #             if not(channel.recv_ready()):
    #                 break

    # sshClient.close()

########################################

    # command = "ls"

    # ssh = paramiko.SSHClient()
    # ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    # ssh.connect(host, port, username, password)
    # while True:
    #     command = input('\n$> ')
    #     stdin, stdout, stderr = ssh.exec_command(command+';\n')
    #     lines = stdout.read()
    #     print(lines.decode())

#########################################

    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(host, username=username, password=password, port=22)

    channel = ssh.invoke_shell()

    out = channel.recv(9999)

    channel.send('cd /mnt\n')
    channel.send('ls\n')

    while not channel.recv_ready():
        pass
    out = channel.recv(9999)
    print(out.decode("ascii"))


import threading
import numpy as np
import imageio
import os
import paramiko
from paramiko import SSHClient
from scp import SCPClient


class HoloeyeSLM (SSHClient):
    class Commands:
        PWD = 'pwd'
        GO_HOME = "cd ~"
        SET_LIBRARY = "export LD_LIBRARY_PATH=/mnt"
        DISABLE_HDMI = "/mnt/ControlExmpl -a"
        SHOW_IMAGE = "/mnt/ControlExmpl -o ~/tmp.bmp"
        CHANGE_IP = "ifconfig usb0 {0} {1}"

    invalid_input_err = "INVALID input: the size of input image/array must be({0},{1}) and the indices/pixels values must be a number between ({2},{3})"
    CACHE_PATH = "./cache"

    '''
    Initializes and establishes the connection to the device
    '''
    def flush_RSA_Keys(self):
        if self.RSA_Keys is None:
            # try the user's .ssh key file, and mask exceptions
            self.RSA_Keys = os.path.expanduser("~/.ssh/known_hosts")
        try:
            fb = open(self.RSA_Keys,'wb')
            fb.write(b'')
            fb.close()
        except IOError:
            pass

    def __init__(self, host='10.10.70.1', port=22, username='root', password='', width=1920, height=1080, min=0, max=255, logging=False,RSA_Keys=None):
        self.logging = logging
        self.RSA_Keys = RSA_Keys
        if(not os.path.exists(self.CACHE_PATH)):
            os.mkdir(self.CACHE_PATH)
        self.width = width
        self.height = height
        self.min = min
        self.max = max
        self.invalid_array_err = self.invalid_input_err.format(
            height, width, min, max)
        self.hostname = host
        self.port = port
        self.username = username
        self.password = password
        self.prepare_connect()

    def prepare_connect(self):
        self.flush_RSA_Keys()
        super().__init__()
        self.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.connect()
        self.channel = self.invoke_shell()
        self._go_home()
        self._set_library_path()
        self.diconnectHDMI()
        self.logger = threading.Thread(target=self.__log,daemon=True,name='HoloeyeSLM {0} Logger'.format(self.hostname))
        self.logger.start()
    
    def __log(self):
        while (self.logging):
            print("Holoeye SLM Log.:. \n")
            while not self.channel.recv_ready():
                pass
            out = self.channel.recv(9999)
            print(out)

    '''
    gets the current session directory
    '''

    def _pwd(self):
        stdin, stdout, stderr = self.exec_command(self.Commands.PWD)
        lines = stdout.readlines()
        return lines

    '''
    Setups the current session directory to home
    '''

    def _go_home(self):
        self.channel.send(self.Commands.GO_HOME+'\n')
        # while not self.channel.recv_ready():
        #         pass
        # out = self.channel.recv(9999)
        # print(out.decode())


    '''
    Setups the libraries path in SLM
    '''

    def _set_library_path(self):
        self.channel.send(self.Commands.SET_LIBRARY+'\n')
        # while not self.channel.recv_ready():
        #         pass
        # out = self.channel.recv(9999)
        # print(out.decode())

    '''
    Runs the show command on slm
    '''

    def _show_image(self):
        self.channel.send(self.Commands.SHOW_IMAGE+'\n')
        # while not self.channel.recv_ready():
        #         pass
        # out = self.channel.recv(9999)
        # print(out.decode())

    '''
    Validates the array whether it is a legitimate input.
    '''

    def _validateArray(self, array):
        if (array.shape == (self.height, self.width)) and (array.max() <= self.max) and (array.min() >= self.min):
            return array
        raise Exception(self.invalid_input_err)

    '''
    Validates the input image path whether it is a legitimate input. And returns the array in case it's a correct format
    '''

    def _validateImage(self, path):
        im = imageio.imread(path)
        return self._validateArray(im)

    '''Saves the image in the temporary path'''

    def _saveImage(self, array):
        try:
            imageio.imwrite(self.CACHE_PATH+'/tmp.bmp', array)
        except Exception as ex:
            print(ex)

    '''Removes the image in the temporary path'''

    def _removeImage(self, array):
        try:
            if os.path.exists(self.CACHE_PATH+'/tmp.bmp'):
                os.remove(self.CACHE_PATH+'/tmp.bmp')
        except Exception as ex:
            print(ex)

    '''
    Establishes connection to the Holoeyes SLM device
    '''

    def connect(self):
        return super().connect(self.hostname, port=self.port, username=self.username, password=self.password)

    '''
    Disoconnects the HDMI from the HoloeyeSLM device
    '''

    def diconnectHDMI(self):
        self.channel.send(self.Commands.DISABLE_HDMI+'\n')
        # while not self.channel.recv_ready():
        #         pass
        # out = self.channel.recv(9999)
        # print(out.decode())

    '''
    Sends an image at a given path to HoloeyeSLM
    '''

    def sendImage(self, FILE_PATH):
        try:
            im = imageio.imread(FILE_PATH)
            array = self._validateImage(FILE_PATH)
            self._saveImage(array)
            # TODO : SEND IMAGE TO HOME DIRECTORY
            with SCPClient(self.get_transport()) as scp:
                scp.put(self.CACHE_PATH+'/tmp.bmp',remote_path='~')
                scp.close()
            self._show_image()
            # TODO : RUN SHOW COMMAND ON

            # self._removeImage()
        except Exception as ex:
            raise ex
    '''
    Sends an nparray to HoloeyeSLM
    '''
    def sendData(self, data):
            try:
                array = self._validateArray(data)
                self._saveImage(array)
                # TODO : SEND IMAGE TO HOME DIRECTORY
                with SCPClient(self.get_transport()) as scp:
                    scp.put(self.CACHE_PATH+'/tmp.bmp',remote_path='~')
                    scp.close()
                self._show_image()
                # TODO : RUN SHOW COMMAND ON
                self._removeImage()
            except Exception as ex:
                raise ex

    '''
    Changes the current HoloeyeSLM IP address
    '''
    def changeIP(self, new_IP='10.10.70.2',mask='255.0.0.0'):
        self.channel.send(self.Commands.CHANGE_IP.format(new_IP,mask)+'\n')
        self.hostname = new_IP



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__all__ = [
    "__packagename__",
    "__version__",
    "__summary__",
    "__url__",
    "__author__",
    "__email__",
    "__license__"
]

# The name of the package 
__packagename__ = "holoeye_embedded"


__version__ = "0.1.0"

# A one-line summary of what the package does. It's like the first line of a doc string. 
__summary__ = "An alternative interface to official API of the Holoeye PLUTO-2 Spatial Light Modulator in order to control multiple devices through RNDIS network rather than classic HDMI, though you don't need to buy multiple vide cards for communicating to all your beloved SLMs."

# The URL of the package's home page 
__url__ = "https://github.com/ehsan2754/HOLOEYE-PLUTO-2-EMBEDDED"

# The name of the author 
__author__ = "Ehsan Shaghaei"

# The email address of the author. (PEP 241 mentions that this might be used as a unique key in some catalog of packages, but I don't know if it actually is.) 
__email__ = "e.shaghaei@innopolis.university"

# PEP 241 says to put the name of the license here, but I don't think that's a good idea. The name doesn't identify a specific license. There are several Zope licenses and several PSF licenses. I recommend putting the URL of the license. 
__license__ = "MIT"
